/* ----------------------------------------------------------------------
   Copyright (C) 2002 Marxmeier Software AG

   Project:  ELOQUENCE
   Module:   dlg / dm
   File:     dlgapi.h
   Function: public dlg
   $Header: /cvs/cvsroot/eloq/B0810/dlgsrv/map/dlgapi.h,v 26.1 2009/05/05 16:28:13 mike Exp $

   NOTE:
      <windows.h> must be included before
      If subsequent include will cause duplicate types and #defines

   This software and documentation is based in part on HP software
   and documentation under license from Hewlett-Packard Company.
   HP is a trademark of Hewlett-Packard Company.
   Copyright 2002 Hewlett-Packard Company.

   Edit History:
   24.10.1993 - creation
   20.01.1994 - merge of hpux / msw version
   06.02.1994 - calling convention, fixed CDECL
   10.02.1994 - dm_dlg_load( PATH )
   28.03.1994 - dm_dlg_load()
   29.08.2002 - set_foreground_window()
   02.11.2007 - int -> LONG, added DLG_BINARY, value.u.bv
   21.12.2007 - fixed win32 compile errors

---------------------------------------------------------------------- */

#ifndef DLGAPI_INCLUDED
#define DLGAPI_INCLUDED


/* ------------------------------------------------------------------------

   API types

------------------------------------------------------------------------ */

#ifndef DLGAPI_TYPES
#define DLGAPI_TYPES

#if defined(WINVER)
/* defined if windows.h is included */
 #define DRVCALL
#else
#if defined(WINDOWS) && !defined(_WIN32)
 #define EXPORT __export
 #define FAR    _far
 #define PASCAL __pascal
 #define CDECL  __cdecl
 #define DRVCALL FAR PASCAL
#else
 #define EXPORT
 #define FAR
 #define PASCAL
 #define CDECL
 #define DRVCALL
#endif
typedef char FAR *LPSTR;
typedef const char FAR *LPCSTR;
#endif

#endif   /* DLGAPI_TYPES */

#ifndef FAR_PASCAL
#define FAR_PASCAL FAR PASCAL
#endif
#ifndef FAR_CDECL
#define FAR_CDECL FAR CDECL
#endif

#ifndef LONG
#define LONG int
#endif


/*
   DLG value container
   ValueType MUST be combatible with eloquence definition
   or the interface code will fail.
*/

enum ValueType {
   DLG_VOID,      /* no value */
   DLG_STRING,    /* string value */
   DLG_INT,       /* integer (32 bit, signed) value */
   DLG_BINARY     /* string value with length */
};

typedef struct Value {
   enum ValueType type;
   union {
      LPCSTR sv;
      LONG iv;
      struct {
         int len;
         LPCSTR ptr;
      } bv;
   } u;
} VALUE;


/* ------------------------------------------------------------------------

   API prototypes

------------------------------------------------------------------------ */

extern int FAR_PASCAL dm_dlg_new(LPCSTR path, LPCSTR classname);
extern int FAR_PASCAL dm_dlg_del(LPCSTR path);
extern int FAR_PASCAL dm_dlg_get(LPCSTR path, VALUE FAR *val);
extern int FAR_PASCAL dm_dlg_set(LPCSTR path, const VALUE FAR *val);
extern int FAR_PASCAL dm_dlg_load(LPCSTR path,LPCSTR dlg,LPCSTR idm,LPCSTR idc,LPCSTR fname);
extern int FAR_PASCAL dm_dlg_draw(LPCSTR path);
extern int FAR_PASCAL dm_dlg_do(LPCSTR path, int options);
extern int FAR_PASCAL dm_dlg_stop(void);
/* extern int FAR_PASCAL dm_do_dlg_load(LPCSTR fname, LPCSTR *p_msg, int FAR *p_line_no); */
extern int FAR_PASCAL dm_dlg_rule(LPCSTR name, LPCSTR path,
                       int argc, const VALUE FAR *argv, VALUE FAR *rval);
extern int FAR_PASCAL dm_dlg_func( LPCSTR name, int argc, const VALUE FAR *argv, VALUE FAR *rval );
extern int FAR_PASCAL dm_dlg_popup( int style, int btn_style, LPCSTR parent,
                         int x, int y, LPCSTR title, LPCSTR text,
                         int btn_cnt, LPCSTR btn[], int def_btn, int FAR *rbtn);
extern void FAR_PASCAL dm_set_popup_focus();
extern int FAR_PASCAL dm_dlg_help( LPCSTR tag );

/*
   Screen update management
 */

enum DM_UPDATE_MODE
{
   UPD_NOW,
   UPD_DELAYED,
   UPD_PENDING
};

extern int dm_update_screen(enum DM_UPDATE_MODE mode);
extern int dm_update_pending(void);

/*
   Utilities
*/

extern int dm_screen_width(void);
extern int dm_screen_height(void);
#ifdef __DMUSER_INCLUDED__
extern int dm_hcenter(DM_ID objID, DM_ID centerID, int *x);
extern int dm_vcenter(DM_ID objID, DM_ID centerID, int *y);
extern int set_foreground_window(DM_ID objID);
extern int make_window_topmost(DM_ID objID);
#endif

/*
   The following prototypes are only visible if IDMuser.h has been included
*/

#ifdef __DMUSER_INCLUDED__

extern void FAR_PASCAL dm_bind_default_callbacks( DM_ID dlg_id );
extern void FAR_PASCAL start_dialog( DM_ID dlgID );
extern void FAR_PASCAL set_active_dialog( DM_ID dlgID );

extern void DML_default DM_ENTRY ExitEventLoop(DM_ID obj, long rule);

extern int FAR_CDECL dm_error_handler(LPCSTR file, int line, int code, LPCSTR msg, ...);
extern long FAR_PASCAL dm_errno(void);

#endif


/*
   this functions must be provided by environment
*/

extern void FAR_CDECL dmdrv_debug(int level, LPCSTR fmt, ...);
extern int  FAR_PASCAL dmdrv_main_loop(long rule, LPCSTR path);

#ifdef __DMUSER_INCLUDED__

extern int FAR_PASCAL dmdrv_attach_dialog(DM_ID objID, long value);
#ifdef DOS
extern DM_ID GetPendingIDMFocusObj(DM_ID parentID);
#endif
extern DM_Boolean DML_default DM_ENTRY HelpOnObject(DM_ID objID);

#endif


/* ------------------------------------------------------------------------

   dlg return codes

------------------------------------------------------------------------ */

enum DialogReturnCodes {
   DLG_OK,           /* success, MUST be zero */
   DLG_FAILED,       /* internal failure */
   DLG_NOFILE,       /* unable to open file */
   DLG_SYNTAX,       /* syntax error in dlg file */
   DLG_NOSPC,        /* no space left in dialog table or MAX_DEPTH exceeded */
   DLG_NOMEM,        /* memory allocation failed */
   DLG_DUPL,         /* duplicate dialog or object name */
   DLG_BADPATH,      /* illegal or bad object path */
   DLG_BADATTR,      /* bad or improper attribute */
   DLG_BADCHILD,     /* parent object does not agree to child */
   DLG_BADTYPE,      /* bad or improper value type */
   DLG_INVAL,        /* invalid value */
   DLG_FOCUS,        /* can't set focus */
   DLG_NOHELP,       /* no help file */
   DLG_NOTAG,        /* no such help tag */
   DLG_MAXERR
};


/* ------------------------------------------------------------------------

   This will map most function names
   Required by m_scan.c

------------------------------------------------------------------------ */

#ifdef DLG_COMPATIBLE_OBSOLETE

#define dlg_map_classname(classname, classrec) \
     dm_dlg_map_classname(classname, classrec)

#define dlg_new(path, classrec) \
     dm_dlg_new(path, classrec)

#define dlg_del(path) \
     dm_dlg_del(path)

#define dlg_get(path, val) \
     dm_dlg_get(path, val)

#define dlg_set( path, val) \
     dm_dlg_set(path, val)

#define dlg_load(fname, p_msg, p_line_no) \
     dm_dlg_load(fname, p_msg, p_line_no)

#define dlg_draw(path) \
     dm_dlg_draw(path)

#define dlg_do(path, options, exit_rule, exit_path) \
     dm_dlg_do(path, options, exit_rule, exit_path)

#define dlg_stop() \
     dm_dlg_stop()

#define dlg_force_redraw( mode ) 0

#endif   /* DLG_COMPATIBLE */

#endif   /* DLGAPI_INCLUDED */
