/*------------------------------------------------------------------------
   Copyright (C) 2002 Marxmeier Software AG

   Project:  ELOQUENCE
   Module:   dlg / dm
   File:     helpview.c
   Function: using vue 3.0 helpview

   This software and documentation is based in part on HP software
   and documentation under license from Hewlett-Packard Company.
   HP is a trademark of Hewlett-Packard Company.
   Copyright 2002 Hewlett-Packard Company.

   Edit history:
   07.01.1994 - creation
   01.03.2000 - HelpOnObject() go to parent on empty tag
   25.10.2000 - Windows: help_dde_module, help_dde_timeout

------------------------------------------------------------------------*/

static char RcsId[] =
   "$Header: /cvs/cvsroot/eloq/B0810/dlgsrv/server/helpview.c,v 26.1 2009/05/05 16:28:14 mike Exp $";


/* ----------------------------------------------------------------------
      includes
---------------------------------------------------------------------- */

#ifdef DOS
# include <windows.h>
#if !defined(WIN32)
# include <toolhelp.h>
#endif
#else
# include <unistd.h>
# include <sys/types.h>
# include <sys/wait.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define DMDRV_PRIVATE
#include "dmdrv.h"

#include <IDMuser.h>
#include "dlgapi.h"
#include "server.h"

extern const DM_ID FAR *const EqDialogID;

#define FAILED_MSG(code,msg) dm_error_handler(__FILE__,__LINE__,code,msg)

#ifdef DOS
/* This will include the DDE management */
# include "ddeclnt.h"
extern char FAR szIniFile[];
#endif

static DM_Boolean do_HelpOnTag(DM_ID objID, LPCSTR tag);
static DM_Boolean OpenURL(LPCSTR url, LPCSTR path, LPCSTR tag, DM_Boolean reload);


/* ----------------------------------------------------------------------

      HelpOnObject

---------------------------------------------------------------------- */

DM_Boolean DML_default DM_ENTRY HelpOnObject(DM_ID objID)
{
   DM_Value data;

   dmdrv_debug(2, "HelpOnObject(obj=%ld)", objID);

   while (   !DM_GetValue(objID, AT_help, 0, &data, DMF_GetMasterString)
          || *data.value.string == '\0')
   {
      /* object does not have .help attribute: get parent */
      if(!DM_GetValue(objID, AT_parent, 0, &data, 0))
      {
         /* no parent: empty tag */
         dmdrv_debug(2, "missing .help attribute, no parent: empty tag");
         return do_HelpOnTag(objID, "");
      }
      objID = data.value.id;
      dmdrv_debug(2, "missing .help attribute, trying parentID=%ld", objID);
   }

   return do_HelpOnTag(objID, data.value.string);
}


/* ----------------------------------------------------------------------

      HelpOnTag

---------------------------------------------------------------------- */

DM_Boolean DML_default DM_ENTRY HelpOnTag(DM_ID objID, LPCSTR tag)
{
   dmdrv_debug(2, "HelpOnTag(obj=%ld, tag=%s)", objID, tag);
   return do_HelpOnTag(objID, tag);
}

static DM_Boolean do_HelpOnTag(DM_ID objID, LPCSTR tag)
{
   DM_ID dlgID, varID;
   DM_Value data;

   if( !objID )
      dlgID = *EqDialogID;
   else {
      if(!DM_GetValue( objID, AT_dialog, 0, &data, 0 )) {
         (void) FAILED_MSG(0, "Unable to obtain dialog ID");
         return(FALSE);
      }
      dlgID = data.value.id;
   }

   varID = DM_DialogPathToID( dlgID, 0, "EqHelpPath" );
   if( !varID ) {
      (void) FAILED_MSG(0, "Unable to obtain EqHelpPath variable ID");
      return FALSE;
   }

   if(!DM_GetValue( varID, AT_value, 0, &data,
                    DMF_GetMasterString | DMF_DontFreeLastStrings )) {
      (void) FAILED_MSG(0, "Unable to obtain EqHelpPath variable value");
      return FALSE;
   }

   return OpenURL(config.help_base_url, data.value.string, tag, 0);
}


/* ----------------------------------------------------------------------

      HelpViewFile / HelpManPage

---------------------------------------------------------------------- */

DM_Boolean DML_default DM_ENTRY HelpViewFile(LPCSTR path)
{
   dmdrv_debug(2, "HelpViewFile(path=%s)", path);
   return OpenURL(config.file_base_url, path, "", 1);
}

DM_Boolean DML_default DM_ENTRY HelpManPage(LPCSTR path)
{
   dmdrv_debug(2, "HelpManPage(path=%s)", path);
   return OpenURL(config.man_base_url, path, "", 0);
}


#ifdef DOS
/* ----------------------------------------------------------------------

      Windows: execute child process

---------------------------------------------------------------------- */

#if !defined(_WIN32)

static int execSuccess(DWORD code)
{
   return code >= 32;
}

static DWORD execute(LPCSTR path, int wait_until_idle)
{
   DWORD code = WinExec(path, SW_SHOW);
   if (wait_until_idle && execSuccess(code))
   {
      TASKENTRY te;

      /* lookup module task ID */
      te.dwSize = (DWORD)sizeof(TASKENTRY);
      if (TaskFirst(&te))
      {
         do {
            if (te.hInst == (HINSTANCE)code)
            {
               /* found: force execution */
               PostAppMessage(te.hTask, WM_NULL, 0, 0);
               DirectedYield(te.hTask);
               break;
            }
         } while (TaskNext(&te));
      }
   }

   return code;
}

#else /* defined(_WIN32) */

static DWORD execute(LPCSTR path, int wait_until_idle)
{
   STARTUPINFO info;
   PROCESS_INFORMATION pi;
   char tmp_path[1024];

   DebugMessage("exec: %s", path);

   memset(&info, 0, sizeof(STARTUPINFO));
   info.cb = sizeof(STARTUPINFO);
   info.dwFlags = STARTF_USESHOWWINDOW;
   info.wShowWindow = SW_SHOW;

   strcpy(tmp_path, path);
   if (!CreateProcess(0, tmp_path, 0, 0, FALSE, 0, 0, 0, &info, &pi))
      return GetLastError();

   CloseHandle(pi.hThread);
   if (wait_until_idle && WaitForInputIdle(pi.hProcess, INFINITE))
   {
      CloseHandle(pi.hProcess);
      return GetLastError();
   }

   CloseHandle(pi.hProcess);

   if (config.help_dde_wait)
      Sleep(config.help_dde_wait);

   return NO_ERROR;
}

static int execSuccess(DWORD code)
{
   return code == NO_ERROR;
}

#endif /* defined(_WIN32) */


/* ----------------------------------------------------------------------

      Windows: perform DDE transaction

---------------------------------------------------------------------- */

static DM_Boolean do_DDE(char *module_p, char *topic_p,
                      char *action_p, char *arg_p, char *value_p,
                      char *result_p, int result_max)
{
   HWND hwndClientDDE, hwndServerDDE;
   int retry, inDDE, getData;
   DWORD execCode;
   char tmp[256];

   dmdrv_debug(2, "do_DDE(module=%s, topic=%s, action=%s, arg=%s, value=%s)",
                  module_p, topic_p, action_p, arg_p, value_p);
   inDDE = FALSE;

   /* initiate DDE connversation */
   InitAckTimeOut(config.help_dde_timeout);
   for (retry = 0;; retry++)
   {
      hwndClientDDE = SendInitiate(module_p, topic_p);
      if (hwndClientDDE == NULL)
      {
         strcpy(tmp, "Could not initiate DDE");
         dmdrv_debug(1, "do_DDE() failed: %s", tmp);
         MessageBox(NULL, tmp,
                    "DDE failed", MB_TASKMODAL | MB_ICONSTOP | MB_OK);
         goto failed;
      }
      inDDE = TRUE;

      /* get DDE server window handle */
      hwndServerDDE = GetHwndServerDDE(hwndClientDDE);
      if (hwndServerDDE != NULL)
         break;

      /* DDE server not found */
      DestroyWindow(hwndClientDDE);
      inDDE = FALSE;

      /* on first try get module file name from INI file if any */
      /* if specified: load module                              */
      if (retry == 0)
      {
         GetPrivateProfileString("modules", module_p, "",
                                 tmp, sizeof(tmp), szIniFile);
         if (*tmp != '\0')
            execCode = execute(tmp, TRUE);
      }

      /* error on second try or if module file name is not */
      /* specified in INI file                             */
      if (retry > 0 || *tmp == '\0' || !execSuccess(execCode))
      {
         sprintf(tmp, "Module \'%s\' is not active"
                      " and I don't know how to start it", module_p);
         dmdrv_debug(1, "do_DDE() failed: %s", tmp);
         MessageBox(NULL, tmp,
                    "DDE failed", MB_TASKMODAL | MB_ICONSTOP | MB_OK);
         goto failed;
      }
   }

   /* do desired action */
   getData = FALSE;
   if (!stricmp(action_p, "execute"))
   {
      SendExecute(hwndClientDDE, hwndServerDDE, arg_p);
   }
   else if (!stricmp(action_p, "request"))
   {
      SendRequest(hwndClientDDE, hwndServerDDE, arg_p);
      getData = TRUE;
   }
   else if (!stricmp(action_p, "poke"))
   {
      SendPoke(hwndClientDDE, hwndServerDDE, arg_p, value_p);
   }
   else {
      sprintf(tmp, "Unknown DDE Action \'%s\'", action_p);
      dmdrv_debug(1, "do_DDE() failed: %s", tmp);
      MessageBox(NULL, tmp,
                 "DDE failed", MB_TASKMODAL | MB_ICONSTOP | MB_OK);
      goto failed;
   }

   if (WaitForAck(hwndClientDDE) != ACK)
   {
      strcpy(tmp, "DDE server did not acknowledge");
      dmdrv_debug(1, "do_DDE() failed: %s", tmp);
      MessageBox(NULL, tmp,
                 "DDE failed", MB_TASKMODAL | MB_ICONSTOP | MB_OK);
      goto failed;
   }

   if (getData)
   {
      if (!GetConvItemValue(hwndClientDDE, arg_p, result_p, result_max))
      {
         strcpy(tmp, "DDE data not available");
         dmdrv_debug(1, "do_DDE() failed: %s", tmp);
         MessageBox(NULL, tmp,
                    "DDE failed", MB_TASKMODAL | MB_ICONSTOP | MB_OK);
         goto failed;
      }
      if (result_p)
         dmdrv_debug(2, "do_DDE() result=[%s]", result_p);
   }

   /* close DDE conversation(s) */
   TerminateConversations();
   return TRUE;

failed:
   if (inDDE)
      TerminateConversations();
   return FALSE;
}
#endif /* DOS */


/* ----------------------------------------------------------------------

      Windows: launch netscape as help viewer via DDE

---------------------------------------------------------------------- */

static DM_Boolean OpenURL(LPCSTR url, LPCSTR path, LPCSTR tag, DM_Boolean reload)
{
   char tmp[1024];
   int rc;

   dmdrv_debug(2, "OpenURL(url=%s, path=%s, tag=%s)", url, path, tag);

#ifdef DOS

   strcpy(tmp, "\"");
   strcat(tmp, url);
   strcat(tmp, path);
   strcat(tmp, tag);
   strcat(tmp, "\",,0xFFFFFFFF,");
   strcat(tmp, reload ? "0x3" : "0x0");

   /* launch NETSCAPE, open URL */
   rc = do_DDE(config.help_dde_module,
               "WWW_OpenURL", "REQUEST", tmp, "", NULL, 0);
   if (!rc)
      return FALSE;

   /* put NETSCAPE on top */
   rc = do_DDE(config.help_dde_module,
               "WWW_Activate", "REQUEST", "0xFFFFFFFF,0x0", "", NULL, 0);
   if (!rc)
      return FALSE;

#else

   if( access(config.netscape, X_OK) == -1 )
   {
      dmdrv_debug(0, "%s: not executable", config.netscape);
      return FALSE;
   }

   strcpy(tmp, config.netscape);
   strcat(tmp, " -remote \"OpenURL(");
   strcat(tmp, url);
   strcat(tmp, path);
   strcat(tmp, tag);
   strcat(tmp, ")\"");

   dmdrv_debug(2, "system(%s)", tmp);
   rc = system(tmp);
   if (!WIFEXITED(rc) || WEXITSTATUS(rc))
      dmdrv_debug(2, "Netscape: rc=0x%x", rc);

   if (WIFEXITED(rc) && WEXITSTATUS(rc) == 1)
   {
      /* netscape not running: start it */
      strcpy(tmp, config.netscape);
      strcat(tmp, " ");
      strcat(tmp, url);
      strcat(tmp, path);
      strcat(tmp, tag);
      strcat(tmp, " &");

      dmdrv_debug(2, "starting Netscape...");
      dmdrv_debug(2, "system(%s)", tmp);
      rc = system(tmp);
      if (!WIFEXITED(rc) || WEXITSTATUS(rc))
         dmdrv_debug(2, "Netscape: rc=0x%x", rc);
   }
   else if (reload && WIFEXITED(rc) && WEXITSTATUS(rc) == 0)
   {
      strcpy(tmp, config.netscape);
      strcat(tmp, " -remote reload");

      dmdrv_debug(2, "system(%s)", tmp);
      rc = system(tmp);
      if (!WIFEXITED(rc) || WEXITSTATUS(rc))
         dmdrv_debug(2, "Netscape: rc=0x%x", rc);
   }

#endif

   return TRUE;
}
